@@ -1,12 +1,12 @@
 from os.path import realpath, join, split
 self_fns = {
 'fr':'entry',
-'st_p':'entry',
-'en_p':'entry',
+'start_percent':'entry',
+'end_percent':'entry',
 'baseline':'entry',
 'brightness':'entry',
 'threshold':'entry',
-'oct_add':'entry',
+'octave_add':'entry',
 'scale_type':'entry',
 'key':'entry',
 'audio_format':'entry',
@@ -16,38 +16,45 @@ self_fns = {
 'file_in':'entry',
 'file_out':'entry',
 'save_path':'entry',
-'Wshow_arr':'value'}
+'Wshow_arr':'value'
+}
 
-oct_add_opts = ['0','1','2','3','4','5']
+octave_add_opts = {
+'0':0,
+'1':12,
+'2':24,
+'3':36,
+'4':48,
+'5':60,
+'6':60
+}
 
-scale_type_opts = ['Chromatic (12/oct)','Major scale (7/oct)','Minor scale (7/oct)', 
-'Maj. triad (3/oct)','Min. triad (3/oct)','Aug. triad (3/oct)',
-'Dim. triad (3/oct)','Maj. 6th (4/oct)','Min. 6th (4/oct)',
-'Maj. 7th (4/oct)','Min. 7th (4/oct)','Aug. 7th (4/oct)',
-'Dim. 7th (4/oct)','Maj. 7/9 (5/oct)','Min. 7/9 (5/oct)']
+scale_keys = {
+'Chromatic (12/oct)':[0,1,2,3,4,5,6,7,8,9,10,11],
+'Major scale (7/oct)':[0,2,4,5,7,9,11],
+'Minor scale (7/oct)':[0,2,3,5,7,9,11],
+'Maj. chord (3/oct)':[0,4,7],
+'Min. chord (3/oct)':[0,3,7],
+'Maj. 7 (4/oct)':[0,4,7,11],
+'Min. 7 (4/oct)':[0,3,7,11],
+'Maj. 2/9 (4/oct)':[0,2,4,7,11],
+'Min. 2/9 (4/oct)':[0,2,3,7,11]
+}
 
-scaledata = [
-[0,1,2,3,4,5,6,7,8,9,10,11],
-[0,2,4,5,7,9,11],
-[0,2,3,5,7,9,11],
-[0,4,7],
-[0,3,7],
-[0,4,8],
-[0,3,6],
-[0,4,7,9 ],
-[0,3,7,9 ],
-[0,4,7,11],
-[0,3,7,11],
-[0,3,7,10],
-[0,4,8,10],
-[0,3,6,9 ],
-[0,3,6,10],
-[0,3,7,11],
-[0,2,4,7,9,11],
-[0,2,3,7,9,11]
-]
-
-key_opts = ['C','C#/D♭','D','D#/E♭','E','F','F#/G♭','G','G#/A♭','A','A#/B♭','B']
+key_opts={
+'C' :0, 
+'C#':1,
+'D' :2,
+'D#':3,
+'E' :4,
+'F' :5,
+'F#':6,
+'G' :7,
+'G#':8,
+'A' :9,
+'A#':10,
+'B' :11
+}
 
 cmaps_opts = tuple(sorted(['viridis', 'plasma', 'inferno', 'magma', 'cividis','binary', 
 'bone', 'pink','spring', 'summer', 'autumn', 'winter', 'cool','hot','copper','Spectral', 
@@ -59,13 +66,19 @@ example_files_decomp = [join(pth,'anthem_datasets',d) for d in ['demo1.mat','dem
 example_files_raw = [join(pth,'anthem_datasets',d) for d in ['raw1.mat','raw2.mat']]
 example_cfg = [join(pth,'anthem_datasets',d) for d in ['demo1_cfg.p','demo2_cfg.p','demo3_cfg.p','demo4_cfg.p']]
 
-soundfonts={
+'''
+Google drive id's for .sf2 files from https://sites.google.com/site/soundfonts4u/
+'''
+sound_fonts={
 'piano_small':'12WYF3pc_kYI5myMjgEUnb-y2CeX3fucx',
 'piano_large':'1DEKLuSiJ4QsHdipfzQdKGsmFn5_4ipOL',
 'e-piano':'0B4_6p-MMrzwLeTBfNzl1SmVVSEU',
 'strings':'1c0pCI0YdcFEpSLEbCW8HTzFOlJpz0HS9'
 }
 
+'''
+static variables
+'''
 
-
-
+C0 = 16.352
+fs = 44100
